@@ -22,7 +22,9 @@ use std::time::Duration;
 use pyo3::prelude::*;
 use pyo3::types::PyBytes;
 use pyo3::types::PyDict;
-use pyo3_asyncio::tokio::future_into_py;
+use pyo3::types::PyTuple;
+use pyo3::IntoPyObjectExt;
+use pyo3_async_runtimes::tokio::future_into_py;
 
 use crate::*;
 
@@ -30,9 +32,9 @@ fn build_operator(
     scheme: ocore::Scheme,
     map: HashMap<String, String>,
 ) -> PyResult<ocore::Operator> {
-    let mut op = ocore::Operator::via_map(scheme, map).map_err(format_pyerr)?;
+    let mut op = ocore::Operator::via_iter(scheme, map).map_err(format_pyerr)?;
     if !op.info().full_capability().blocking {
-        let runtime = pyo3_asyncio::tokio::get_runtime();
+        let runtime = pyo3_async_runtimes::tokio::get_runtime();
         let _guard = runtime.enter();
         op = op
             .layer(ocore::layers::BlockingLayer::create().expect("blocking layer must be created"));
@@ -45,7 +47,11 @@ fn build_operator(
 ///
 /// Create a new blocking `Operator` with the given `scheme` and options(`**kwargs`).
 #[pyclass(module = "opendal")]
-pub struct Operator(ocore::BlockingOperator);
+pub struct Operator {
+    core: ocore::BlockingOperator,
+    __scheme: ocore::Scheme,
+    __map: HashMap<String, String>,
+}
 
 #[pymethods]
 impl Operator {
@@ -65,18 +71,26 @@ impl Operator {
             })
             .unwrap_or_default();
 
-        Ok(Operator(build_operator(scheme, map)?.blocking()))
+        Ok(Operator {
+            core: build_operator(scheme.clone(), map.clone())?.blocking(),
+            __scheme: scheme,
+            __map: map,
+        })
     }
 
     /// Add new layers upon existing operator
     pub fn layer(&self, layer: &layers::Layer) -> PyResult<Self> {
-        let op = layer.0.layer(self.0.clone().into());
-        Ok(Self(op.blocking()))
+        let op = layer.0.layer(self.core.clone().into());
+        Ok(Self {
+            core: op.blocking(),
+            __scheme: self.__scheme.clone(),
+            __map: self.__map.clone(),
+        })
     }
 
     /// Open a file-like reader for the given path.
     pub fn open(&self, path: String, mode: String) -> PyResult<File> {
-        let this = self.0.clone();
+        let this = self.core.clone();
         if mode == "rb" {
             let r = this
                 .reader(&path)
@@ -88,15 +102,15 @@ impl Operator {
             let w = this.writer(&path).map_err(format_pyerr)?;
             Ok(File::new_writer(w))
         } else {
-            Err(UnsupportedError::new_err(format!(
+            Err(Unsupported::new_err(format!(
                 "OpenDAL doesn't support mode: {mode}"
             )))
         }
     }
 
     /// Read the whole path into bytes.
-    pub fn read<'p>(&'p self, py: Python<'p>, path: &str) -> PyResult<Bound<PyAny>> {
-        let buffer = self.0.read(path).map_err(format_pyerr)?.to_vec();
+    pub fn read<'p>(&'p self, py: Python<'p>, path: &str) -> PyResult<Bound<'p, PyAny>> {
+        let buffer = self.core.read(path).map_err(format_pyerr)?.to_vec();
         Buffer::new(buffer).into_bytes_ref(py)
     }
 
@@ -104,7 +118,10 @@ impl Operator {
     #[pyo3(signature = (path, bs, **kwargs))]
     pub fn write(&self, path: &str, bs: Vec<u8>, kwargs: Option<WriteOptions>) -> PyResult<()> {
         let kwargs = kwargs.unwrap_or_default();
-        let mut write = self.0.write_with(path, bs).append(kwargs.append);
+        let mut write = self
+            .core
+            .write_with(path, bs)
+            .append(kwargs.append.unwrap_or(false));
         if let Some(chunk) = kwargs.chunk {
             write = write.chunk(chunk);
         }
@@ -123,22 +140,25 @@ impl Operator {
 
     /// Get current path's metadata **without cache** directly.
     pub fn stat(&self, path: &str) -> PyResult<Metadata> {
-        self.0.stat(path).map_err(format_pyerr).map(Metadata::new)
+        self.core
+            .stat(path)
+            .map_err(format_pyerr)
+            .map(Metadata::new)
     }
 
     /// Copy source to target.
     pub fn copy(&self, source: &str, target: &str) -> PyResult<()> {
-        self.0.copy(source, target).map_err(format_pyerr)
+        self.core.copy(source, target).map_err(format_pyerr)
     }
 
     /// Rename filename.
     pub fn rename(&self, source: &str, target: &str) -> PyResult<()> {
-        self.0.rename(source, target).map_err(format_pyerr)
+        self.core.rename(source, target).map_err(format_pyerr)
     }
 
     /// Remove all file
     pub fn remove_all(&self, path: &str) -> PyResult<()> {
-        self.0.remove_all(path).map_err(format_pyerr)
+        self.core.remove_all(path).map_err(format_pyerr)
     }
 
     /// Create a dir at given path.
@@ -154,7 +174,7 @@ impl Operator {
     /// - Create on existing dir will succeed.
     /// - Create dir is always recursive, works like `mkdir -p`
     pub fn create_dir(&self, path: &str) -> PyResult<()> {
-        self.0.create_dir(path).map_err(format_pyerr)
+        self.core.create_dir(path).map_err(format_pyerr)
     }
 
     /// Delete given path.
@@ -163,19 +183,19 @@ impl Operator {
     ///
     /// - Delete not existing error won't return errors.
     pub fn delete(&self, path: &str) -> PyResult<()> {
-        self.0.delete(path).map_err(format_pyerr)
+        self.core.delete(path).map_err(format_pyerr)
     }
 
     /// List current dir path.
     pub fn list(&self, path: &str) -> PyResult<BlockingLister> {
-        let l = self.0.lister(path).map_err(format_pyerr)?;
+        let l = self.core.lister(path).map_err(format_pyerr)?;
         Ok(BlockingLister::new(l))
     }
 
     /// List dir in flat way.
     pub fn scan(&self, path: &str) -> PyResult<BlockingLister> {
         let l = self
-            .0
+            .core
             .lister_with(path)
             .recursive(true)
             .call()
@@ -184,15 +204,21 @@ impl Operator {
     }
 
     pub fn capability(&self) -> PyResult<capability::Capability> {
-        Ok(capability::Capability::new(self.0.info().full_capability()))
+        Ok(capability::Capability::new(
+            self.core.info().full_capability(),
+        ))
     }
 
     pub fn to_async_operator(&self) -> PyResult<AsyncOperator> {
-        Ok(AsyncOperator(self.0.clone().into()))
+        Ok(AsyncOperator {
+            core: self.core.clone().into(),
+            __scheme: self.__scheme.clone(),
+            __map: self.__map.clone(),
+        })
     }
 
     fn __repr__(&self) -> String {
-        let info = self.0.info();
+        let info = self.core.info();
         let name = info.name();
         if name.is_empty() {
             format!("Operator(\"{}\", root=\"{}\")", info.scheme(), info.root())
@@ -204,13 +230,24 @@ impl Operator {
             )
         }
     }
+
+    fn __getnewargs_ex__(&self, py: Python) -> PyResult<PyObject> {
+        let args = vec![self.__scheme.to_string()];
+        let args = PyTuple::new(py, args)?.into_py_any(py)?;
+        let kwargs = self.__map.clone().into_py_any(py)?;
+        Ok(PyTuple::new(py, [args, kwargs])?.into_py_any(py)?)
+    }
 }
 
 /// `AsyncOperator` is the entry for all public async APIs
 ///
 /// Create a new `AsyncOperator` with the given `scheme` and options(`**kwargs`).
 #[pyclass(module = "opendal")]
-pub struct AsyncOperator(ocore::Operator);
+pub struct AsyncOperator {
+    core: ocore::Operator,
+    __scheme: ocore::Scheme,
+    __map: HashMap<String, String>,
+}
 
 #[pymethods]
 impl AsyncOperator {
@@ -230,13 +267,21 @@ impl AsyncOperator {
             })
             .unwrap_or_default();
 
-        Ok(AsyncOperator(build_operator(scheme, map)?))
+        Ok(AsyncOperator {
+            core: build_operator(scheme.clone(), map.clone())?.into(),
+            __scheme: scheme,
+            __map: map,
+        })
     }
 
     /// Add new layers upon existing operator
     pub fn layer(&self, layer: &layers::Layer) -> PyResult<Self> {
-        let op = layer.0.layer(self.0.clone());
-        Ok(Self(op))
+        let op = layer.0.layer(self.core.clone());
+        Ok(Self {
+            core: op,
+            __scheme: self.__scheme.clone(),
+            __map: self.__map.clone(),
+        })
     }
 
     /// Open a file-like reader for the given path.
@@ -245,8 +290,8 @@ impl AsyncOperator {
         py: Python<'p>,
         path: String,
         mode: String,
-    ) -> PyResult<Bound<PyAny>> {
-        let this = self.0.clone();
+    ) -> PyResult<Bound<'p, PyAny>> {
+        let this = self.core.clone();
 
         future_into_py(py, async move {
             if mode == "rb" {
@@ -262,7 +307,7 @@ impl AsyncOperator {
                 let w = this.writer(&path).await.map_err(format_pyerr)?;
                 Ok(AsyncFile::new_writer(w))
             } else {
-                Err(UnsupportedError::new_err(format!(
+                Err(Unsupported::new_err(format!(
                     "OpenDAL doesn't support mode: {mode}"
                 )))
             }
@@ -270,8 +315,8 @@ impl AsyncOperator {
     }
 
     /// Read the whole path into bytes.
-    pub fn read<'p>(&'p self, py: Python<'p>, path: String) -> PyResult<Bound<PyAny>> {
-        let this = self.0.clone();
+    pub fn read<'p>(&'p self, py: Python<'p>, path: String) -> PyResult<Bound<'p, PyAny>> {
+        let this = self.core.clone();
         future_into_py(py, async move {
             let res: Vec<u8> = this.read(&path).await.map_err(format_pyerr)?.to_vec();
             Python::with_gil(|py| Buffer::new(res).into_bytes(py))
@@ -286,12 +331,14 @@ impl AsyncOperator {
         path: String,
         bs: &Bound<PyBytes>,
         kwargs: Option<WriteOptions>,
-    ) -> PyResult<Bound<PyAny>> {
+    ) -> PyResult<Bound<'p, PyAny>> {
         let kwargs = kwargs.unwrap_or_default();
-        let this = self.0.clone();
+        let this = self.core.clone();
         let bs = bs.as_bytes().to_vec();
         future_into_py(py, async move {
-            let mut write = this.write_with(&path, bs).append(kwargs.append);
+            let mut write = this
+                .write_with(&path, bs)
+                .append(kwargs.append.unwrap_or(false));
             if let Some(buffer) = kwargs.chunk {
                 write = write.chunk(buffer);
             }
@@ -309,8 +356,8 @@ impl AsyncOperator {
     }
 
     /// Get current path's metadata **without cache** directly.
-    pub fn stat<'p>(&'p self, py: Python<'p>, path: String) -> PyResult<Bound<PyAny>> {
-        let this = self.0.clone();
+    pub fn stat<'p>(&'p self, py: Python<'p>, path: String) -> PyResult<Bound<'p, PyAny>> {
+        let this = self.core.clone();
         future_into_py(py, async move {
             let res: Metadata = this
                 .stat(&path)
@@ -328,8 +375,8 @@ impl AsyncOperator {
         py: Python<'p>,
         source: String,
         target: String,
-    ) -> PyResult<Bound<PyAny>> {
-        let this = self.0.clone();
+    ) -> PyResult<Bound<'p, PyAny>> {
+        let this = self.core.clone();
         future_into_py(py, async move {
             this.copy(&source, &target).await.map_err(format_pyerr)
         })
@@ -341,16 +388,16 @@ impl AsyncOperator {
         py: Python<'p>,
         source: String,
         target: String,
-    ) -> PyResult<Bound<PyAny>> {
-        let this = self.0.clone();
+    ) -> PyResult<Bound<'p, PyAny>> {
+        let this = self.core.clone();
         future_into_py(py, async move {
             this.rename(&source, &target).await.map_err(format_pyerr)
         })
     }
 
     /// Remove all file
-    pub fn remove_all<'p>(&'p self, py: Python<'p>, path: String) -> PyResult<Bound<PyAny>> {
-        let this = self.0.clone();
+    pub fn remove_all<'p>(&'p self, py: Python<'p>, path: String) -> PyResult<Bound<'p, PyAny>> {
+        let this = self.core.clone();
         future_into_py(py, async move {
             this.remove_all(&path).await.map_err(format_pyerr)
         })
@@ -368,8 +415,8 @@ impl AsyncOperator {
     ///
     /// - Create on existing dir will succeed.
     /// - Create dir is always recursive, works like `mkdir -p`
-    pub fn create_dir<'p>(&'p self, py: Python<'p>, path: String) -> PyResult<Bound<PyAny>> {
-        let this = self.0.clone();
+    pub fn create_dir<'p>(&'p self, py: Python<'p>, path: String) -> PyResult<Bound<'p, PyAny>> {
+        let this = self.core.clone();
         future_into_py(py, async move {
             this.create_dir(&path).await.map_err(format_pyerr)
         })
@@ -380,8 +427,8 @@ impl AsyncOperator {
     /// # Notes
     ///
     /// - Delete not existing error won't return errors.
-    pub fn delete<'p>(&'p self, py: Python<'p>, path: String) -> PyResult<Bound<PyAny>> {
-        let this = self.0.clone();
+    pub fn delete<'p>(&'p self, py: Python<'p>, path: String) -> PyResult<Bound<'p, PyAny>> {
+        let this = self.core.clone();
         future_into_py(
             py,
             async move { this.delete(&path).await.map_err(format_pyerr) },
@@ -389,25 +436,27 @@ impl AsyncOperator {
     }
 
     /// List current dir path.
-    pub fn list<'p>(&'p self, py: Python<'p>, path: String) -> PyResult<Bound<PyAny>> {
-        let this = self.0.clone();
+    pub fn list<'p>(&'p self, py: Python<'p>, path: String) -> PyResult<Bound<'p, PyAny>> {
+        let this = self.core.clone();
         future_into_py(py, async move {
             let lister = this.lister(&path).await.map_err(format_pyerr)?;
-            let pylister: PyObject = Python::with_gil(|py| AsyncLister::new(lister).into_py(py));
+            let pylister = Python::with_gil(|py| AsyncLister::new(lister).into_py_any(py))?;
+
             Ok(pylister)
         })
     }
 
     /// List dir in flat way.
-    pub fn scan<'p>(&'p self, py: Python<'p>, path: String) -> PyResult<Bound<PyAny>> {
-        let this = self.0.clone();
+    pub fn scan<'p>(&'p self, py: Python<'p>, path: String) -> PyResult<Bound<'p, PyAny>> {
+        let this = self.core.clone();
         future_into_py(py, async move {
             let lister = this
                 .lister_with(&path)
                 .recursive(true)
                 .await
                 .map_err(format_pyerr)?;
-            let pylister: PyObject = Python::with_gil(|py| AsyncLister::new(lister).into_py(py));
+            let pylister: PyObject =
+                Python::with_gil(|py| AsyncLister::new(lister).into_py_any(py))?;
             Ok(pylister)
         })
     }
@@ -418,8 +467,8 @@ impl AsyncOperator {
         py: Python<'p>,
         path: String,
         expire_second: u64,
-    ) -> PyResult<Bound<PyAny>> {
-        let this = self.0.clone();
+    ) -> PyResult<Bound<'p, PyAny>> {
+        let this = self.core.clone();
         future_into_py(py, async move {
             let res = this
                 .presign_stat(&path, Duration::from_secs(expire_second))
@@ -437,8 +486,8 @@ impl AsyncOperator {
         py: Python<'p>,
         path: String,
         expire_second: u64,
-    ) -> PyResult<Bound<PyAny>> {
-        let this = self.0.clone();
+    ) -> PyResult<Bound<'p, PyAny>> {
+        let this = self.core.clone();
         future_into_py(py, async move {
             let res = this
                 .presign_read(&path, Duration::from_secs(expire_second))
@@ -456,8 +505,8 @@ impl AsyncOperator {
         py: Python<'p>,
         path: String,
         expire_second: u64,
-    ) -> PyResult<Bound<PyAny>> {
-        let this = self.0.clone();
+    ) -> PyResult<Bound<'p, PyAny>> {
+        let this = self.core.clone();
         future_into_py(py, async move {
             let res = this
                 .presign_write(&path, Duration::from_secs(expire_second))
@@ -470,15 +519,21 @@ impl AsyncOperator {
     }
 
     pub fn capability(&self) -> PyResult<capability::Capability> {
-        Ok(capability::Capability::new(self.0.info().full_capability()))
+        Ok(capability::Capability::new(
+            self.core.info().full_capability(),
+        ))
     }
 
     pub fn to_operator(&self) -> PyResult<Operator> {
-        Ok(Operator(self.0.clone().blocking()))
+        Ok(Operator {
+            core: self.core.clone().blocking(),
+            __scheme: self.__scheme.clone(),
+            __map: self.__map.clone(),
+        })
     }
 
     fn __repr__(&self) -> String {
-        let info = self.0.info();
+        let info = self.core.info();
         let name = info.name();
         if name.is_empty() {
             format!(
@@ -494,6 +549,13 @@ impl AsyncOperator {
             )
         }
     }
+
+    fn __getnewargs_ex__(&self, py: Python) -> PyResult<PyObject> {
+        let args = vec![self.__scheme.to_string()];
+        let args = PyTuple::new(py, args)?.into_py_any(py)?;
+        let kwargs = self.__map.clone().into_py_any(py)?;
+        Ok(PyTuple::new(py, [args, kwargs])?.into_py_any(py)?)
+    }
 }
 
 #[pyclass(module = "opendal")]
@@ -521,9 +583,9 @@ impl PresignedRequest {
             let k = k.as_str();
             let v = v
                 .to_str()
-                .map_err(|err| UnexpectedError::new_err(err.to_string()))?;
+                .map_err(|err| Unexpected::new_err(err.to_string()))?;
             if headers.insert(k, v).is_some() {
-                return Err(UnexpectedError::new_err("duplicate header"));
+                return Err(Unexpected::new_err("duplicate header"));
             }
         }
         Ok(headers)
